/*
  Copyright 2008-2014 VLSI Solution Oy. Absolutely no warranty.
*/  

#include "system.h"

#include <stdio.h> //Standard io
#include <stdlib.h> // VS_DSP Standard Library
#include <vs1000.h> // VS1000B register definitions
#include <vectors.h> // VS1000B vectors (interrupts and services)
#include <minifat.h> // Read Only Fat Filesystem
#include <mapper.h> // Logical Disk
#include <string.h> // memcpy etc
#include <player.h> // VS1000B default ROM player
#include <audio.h> // DAC output
#include <codec.h> // CODEC interface
#include <vsNand.h>
#include <mappertiny.h>
#include <usb.h>

#include <dev1000.h>

extern struct Codec *cod;
extern struct CodecServices cs;

#ifdef LOOP_MODE /* only with gapless */
/* For seamless playing of the same file in a loop. These are
   not normally used, so we don't bother including codecvorbis.h . */
auto s_int16 CodVorbisGoTo(register struct CodecVorbis *cv,
        		   register u_int16 seconds);
int loopMode = 0, leaveLoop = 0;
#endif/*LOOP_MODE*/

#if defined(GAPLESS) || defined(LOOP_MODE)
struct Codec *CodVorbisCreate(void);
extern u_int16 ogg[];
//extern struct CodVOgg ogg;
extern s_int16 vFirstFrame;
auto s_int16 OggSeekHeader(register u_int32 pos);
#endif/*GAPLESS*/








#if defined(GAPLESS) && defined(LOOP_MODE)
#include <codecmicrowav.h>
#include <dev1000.h>
enum CodecError PlayGapless(void) {
    u_int16 ret = ceOtherError;
    const char *eStr;
    /*If you use fixed clock (with UART), you can comment out the LoadChecks.*/
    LoadCheck(NULL, 0); /* higher clock -- if not already */
    if (codecVorbis.audioBegins) {
        u_int16 data[2];
        Seek(codecVorbis.audioBegins);
        ReadFile(data, 0, 4);
        Seek(0); //Seek(0) keeps original fileLeft correct
        if (data[0] == 0x4f67 && data[1] == 0x6753) {
            /* 'OggS', assume the same parameters,
               skip Vorbis decoder init. */
            OggSeekHeader(codecVorbis.audioBegins);
            ogg[1]/*.headerTypeFlag*/ = 0;
            ogg[6]/*.streamSerialNumber*/ = 0;
            ogg[7]/*.streamSerialNumber*/ = 0;
            ogg[14]/*.cont*/ = 0;
            vFirstFrame = 1;
            cs.playTimeSeconds = 0;
            cs.playTimeSamples = 0;
            /*Start decoding without recreating the decoding tables
              from header.*/
putch('g');
        } else {
            /* 'OggS' Not found, must be a file with different parameters
               (or WAV) */
        normal:
#ifdef USE_WAV
            //CodMicroWavCreate(); /*We know it does nothing */
            /* call Decode directly to save some instructions */
            ret = CodMicroWavDecode(cod, &cs, &eStr);
            //CodMicroWavDelete(cod); /*We know it does nothing */
            if (ret != ceFormatNotFound) {
#ifdef LOOP_MODE
        	leaveLoop = 0;
        	while (loopMode && !leaveLoop && !cs.cancel /*cs.fileLeft < 5*//*arbitrary end limit -- does not work well with WAV */) {
//putch('l');
        	    LoadCheck(NULL, 0); /*higher clock*/
        	    cs.Seek(&cs, 0, SEEK_SET); /* seek to the beginning */
        	    CodMicroWavDecode(cod, &cs, &eStr); /* and decode again */
#ifdef GAPLESS
        	    /* Must clear this after using the vorbis area! */
        	    codecVorbis.audioBegins = 0;
#endif/*GAPLESS*/
        	    IdleHook(); //so that it is called sometime!
        	}
#endif/*LOOP_MODE*/
        	return ret;
            }
            /* If failed, seek to the beginning and try Ogg Vorbis decoding. */
            cs.Seek(&cs, 0, SEEK_SET);
#endif/*USE_WAV*/
            if (!(cod = CodVorbisCreate())) {
        	return ceOtherError; //should never happen
            }
            //putch('-');
        }
        /* Decode the file once */
#ifdef HAS_PATCHCODVORBISDECODE
        /*available in latest dev1000.h */
        ret = PatchCodVorbisDecode(&codecVorbis, &cs, &eStr, 0); 
#else
        ret = CodVorbisDecode(&codecVorbis, &cs, &eStr); 
#endif
#ifdef LOOP_MODE
        /* If loop mode, and played the file to the end.
           Clearing loop mode in the middle will play a repeat to the end,
           then return from play.
         */
        leaveLoop = 0;
        while (loopMode && !leaveLoop && cs.fileLeft < 5/*arbitrary end limit*/) {
//putch('l');
            LoadCheck(NULL, 0); /*higher clock*/
            /* Perform the minimal clearing */
            ogg[1]/*headerTypeFlag*/ = 0;
            CodVorbisGoTo(cod, 0);

            ogg[1]/*.headerTypeFlag*/ = 0;
            ogg[14]/*.cont*/ = 0;
            vFirstFrame = 1; /* skip the first windowed samples */

#ifdef HAS_PATCHCODVORBISDECODE
            /*available in latest dev1000.h */
            ret = PatchCodVorbisDecode(&codecVorbis, &cs, &eStr, 0); 
#else
            ret = CodVorbisDecode(&codecVorbis, &cs, &eStr); 
#endif
            IdleHook(); //so that it is called sometime!
        }
#endif /*LOOP_MODE*/
        //cod->Delete(cod); /* does not do anything */
    } else {
        goto normal;
    }
    return ret;
}
#endif/*GAPLESS*/


#if !defined(GAPLESS) && defined(LOOP_MODE)
#include <codecmicrowav.h>
#include <dev1000.h>
enum CodecError PlayWavOrOggFile(void) {
    u_int16 ret = ceOtherError;
    const char *eStr;
    /*If you use fixed clock (with UART), you can comment out the LoadChecks.*/
    LoadCheck(NULL, 0); /* higher clock -- if not already */
#ifdef USE_WAV
    //CodMicroWavCreate(); /*We know it does nothing */
    /* call Decode directly to save some instructions */
    ret = CodMicroWavDecode(cod, &cs, &eStr);
    //CodMicroWavDelete(cod); /*We know it does nothing */
    if (ret != ceFormatNotFound) {
      leaveLoop = 0;
      while (loopMode && !leaveLoop && !cs.cancel /*cs.fileLeft < 5*//*arbitrary end limit -- does not work well with WAV */) {
        //putch('l');
        LoadCheck(NULL, 0); /*higher clock*/
        cs.Seek(&cs, 0, SEEK_SET); /* seek to the beginning */
        CodMicroWavDecode(cod, &cs, &eStr); /* and decode again */
        IdleHook(); //so that it is called sometime!
      }
      return ret;
    }
    /* If failed, seek to the beginning and try Ogg Vorbis decoding. */
    cs.Seek(&cs, 0, SEEK_SET);
#endif/*USE_WAV*/
    if (!(cod = CodVorbisCreate())) {
      return ceOtherError; //should never happen
    }
    putch('-');
    /* Decode the file once */
#ifdef HAS_PATCHCODVORBISDECODE
    /*available in latest dev1000.h */
    ret = PatchCodVorbisDecode(&codecVorbis, &cs, &eStr, 0); 
#else
    ret = CodVorbisDecode(&codecVorbis, &cs, &eStr); 
#endif
    /* If loop mode, and played the file to the end.
       Clearing loop mode in the middle will play a repeat to the end,
       then return from play.
    */
    leaveLoop = 0;
    while (loopMode && !leaveLoop && cs.fileLeft < 5/*arbitrary end limit*/) {
//putch('l');
      LoadCheck(NULL, 0); /*higher clock*/
      /* Perform the minimal clearing */
      ogg[1]/*headerTypeFlag*/ = 0;
      CodVorbisGoTo(cod, 0);

      ogg[1]/*.headerTypeFlag*/ = 0;
      ogg[14]/*.cont*/ = 0;
      vFirstFrame = 1; /* skip the first windowed samples */

#ifdef HAS_PATCHCODVORBISDECODE
      /*available in latest dev1000.h */
      ret = PatchCodVorbisDecode(&codecVorbis, &cs, &eStr, 0); 
#else
      ret = CodVorbisDecode(&codecVorbis, &cs, &eStr); 
#endif
      IdleHook(); //so that it is called sometime!
    }
    //cod->Delete(cod); /* does not do anything */
    return ret;
}
#endif/*!GAPLESS LOOP_MODE*/










#if defined(USE_WAV) && !defined(LOOP_MODE)
#include <codecmicrowav.h>
#include <dev1000.h>
//extern u_int16 codecVorbis[];
//extern u_int16 ogg[];
//extern u_int16 mInt[];

extern struct CodecServices cs;
extern struct Codec *cod;
enum CodecError PlayWavOrOggFile(void) {
  register enum CodecError ret = ceFormatNotFound;
  const char *eStr;
  LoadCheck(NULL, 0); /* higher clock, but 4.0x not absolutely required */

#ifdef GAPLESS
  if (codecVorbis.audioBegins) {
      u_int16 data[2];
      Seek(codecVorbis.audioBegins);
      ReadFile(data, 0, 4);
      if (data[0] == 0x4f67 && data[1] == 0x6753) {
          /* 'OggS', assume the same parameters, skip
             Vorbis decoder init. */
          const char *eStr;
          register s_int16 res = OggSeekHeader(codecVorbis.audioBegins);
          ogg[1]/*.headerTypeFlag*/ = 0;
          ogg[6]/*.streamSerialNumber*/ = 0;
          ogg[7]/*.streamSerialNumber*/ = 0;
          ogg[14]/*.cont*/ = 0;
          //ogg.headerTypeFlag = 0;
          //ogg.cont = 0;
          vFirstFrame = 1;
          cs.playTimeSeconds = 0;
          cs.playTimeSamples = 0;
//putch('+');
#ifdef HAS_PATCHCODVORBISDECODE
          /*available in latest dev1000.h */
          return PatchCodVorbisDecode(&codecVorbis, &cs, &eStr, 0); 
#else
          /*TODO: call Decode() with silentchannelpatches */
          return CodVorbisDecode(&codecVorbis, &cs, &eStr); 
#endif
      } else {
          /* 'OggS' Not found, must be a file with
             different parameters (or WAV) */
//putch('-');
          Seek(0);
      }
  }
#endif/*GAPLESS*/

  if ((cod = CodMicroWavCreate())) {
      ret = cod->Decode(cod, &cs, &eStr);
      cod->Delete(cod);
#if 0
      if (earSpeakerReg) {
          register int i;
          for (i=0;i<DEFAULT_AUDIO_BUFFER_SAMPLES/(sizeof(tmpBuf)/2);i++) {
              /* Must be memset each time, because effects like EarSpeaker are
                 performed in-place in the buffer */
              memset(tmpBuf, 0, sizeof(tmpBuf));
              AudioOutputSamples(tmpBuf, sizeof(tmpBuf)/2);
          }
      }
#endif
       if (ret != ceFormatNotFound) {
#ifdef GAPLESS
          codecVorbis.audioBegins = 0;
#endif
          return ret;
      }
      /* If failed, seek to the beginning and try Ogg Vorbis decoding. */
      cs.Seek(&cs, 0, SEEK_SET);
  }
  return PatchPlayCurrentFile();
}
#else/*USE_WAV !LOOP_MODE*/

#if defined(GAPLESS) && !defined(LOOP_MODE)
enum CodecError PlayGaplessOggFile(void) {
    if (codecVorbis.audioBegins) {
        u_int16 data[2];

        Seek(codecVorbis.audioBegins);
        ReadFile(data, 0, 4);
        if (data[0] == 0x4f67 && data[1] == 0x6753) {
            /* 'OggS', assume the same parameters, skip
               Vorbis decoder init. */
            const char *eStr;
            register s_int16 res = OggSeekHeader(codecVorbis.audioBegins);
            ogg[1]/*.headerTypeFlag*/ = 0;
            ogg[6]/*.streamSerialNumber*/ = 0;
            ogg[7]/*.streamSerialNumber*/ = 0;
            ogg[14]/*.cont*/ = 0;
            //ogg.headerTypeFlag = 0;
            //ogg.cont = 0;
            vFirstFrame = 1;
            cs.playTimeSeconds = 0;
            cs.playTimeSamples = 0;
/*Start decoding without recreating the decoding tables from header.*/
//putch('+');
#ifdef HAS_PATCHCODVORBISDECODE
            /*available in latest dev1000.h */
            return PatchCodVorbisDecode(&codecVorbis, &cs, &eStr, 0); 
#else
            return CodVorbisDecode(&codecVorbis, &cs, &eStr); 
#endif
        }
        /* 'OggS' Not found, must be a file with
           different parameters (or WAV) */
//putch('-');
        Seek(0);
        return PatchPlayCurrentFile();
    }
//putch('n');
    return PatchPlayCurrentFile();
}
#endif/*GAPLESS*/
#endif/*elseUSE_WAV*/
